import click
import json
import time
import datetime

from horuz.cli import pass_environment
from horuz.utils.formatting import beautify_query
from horuz.utils.es import HoruzES
from horuz.utils.style import rtable


@click.command("search", short_help="Search data in ES.")
@click.option("-v", "--verbose", is_flag=True, help="Enables verbose mode.")
@click.option('-p', '--project', required=True, help='Project name. Example misalabs.com x.com...')
@click.option('-q', '--query', required=True, help='Query to ElasticSeach')
@click.option('-f', '--fields', help='Specify the fields you want.')
@click.option('-s', '--size', default=100, type=click.IntRange(1, 10000), help='Specify the output size. Range 1-10000')
@click.option('-o', '--order', default="time:desc", help='Specify the sorting of the query. Default time:desc')
@click.option('-oJ', is_flag=True, help="JSON Output")
@click.option('-tl', '--tail', is_flag=True, help="Get the last live info from ElasticSearch. Based on your custom order flag.")
@pass_environment
def cli(ctx, verbose, project, query, fields, size, order, oj, tail):
    """
    Get data from ElasticSeach.
    """
    ctx.verbose = verbose
    hes = HoruzES(project, ctx)
    fields = fields.split(",") if fields else []
    if oj:
        # JSON Output
        data = beautify_query(
            hes.query(term=query, size=size, order=order, fields=fields),
            fields,
            output="json")
        click.echo(data)
    elif tail:
        # Get the last infor from elasticsearch
        if not fields:
            fields = ["_id", "time", "session"]
        showed_ids = []
        while True:
            data = beautify_query(
                hes.query(term=query, size=1, order=order, fields=fields),
                fields,
                output="interactive")
            if data and data[0]['_id'] not in showed_ids:
                showed_ids.append(data[0]['_id'])
                click.echo(json.dumps(data[0]))
            time.sleep(2)
    else:
        # Interactive Output
        # Default fields if nothing were introduced
        if not fields:
            fields = ["_id", "time", "session"]
        data = beautify_query(
            hes.query(term=query, size=size, order=order, fields=fields),
            fields,
            output="interactive")
        # Adding columns
        if data:
            for column in data[0].keys():
                rtable.add_column(column, style="cyan")
            for i in data:
                rtable.add_row(*i.values())
            ctx.log(rtable, pager=True)



import codecs
import json
import traceback


def recursive_items(dictionary):
    for key, value in sorted(dictionary.items()):
        if type(value) is dict:
            yield from recursive_items(value)
        else:
            yield (key, value)


def beautify_query(query, fields=[], output="oj"):
    """
    Prepare the query for the user.
    Parameters
    ----------
    query : ElasticSearch Query
    fields : Fields that nees to be in the result
    output : JSON, Interactive
    """
    data = []
    try:
        if query and query['hits']:
            for hit in query['hits']['hits']:
                source = hit["_source"]
                d = source
                d["_id"] = hit["_id"]
                data.append(d)
    except Exception as e:
        raise ValueError("""
            Query term is malformed.
            Exception: %r
            Traceback: %s
            """ % (e, traceback.format_exc()))

    if output == "json":
        data = json.dumps(data, indent=4, sort_keys=True)
    elif output == "interactive":
        new_data = []
        for d in data:
            new_d = {}
            for key, value in recursive_items(d):
                new_d[key] = codecs.decode(str(value), "unicode_escape")
            new_data.append(new_d)
        data = new_data
    return data



import os
import sys

import click
from .utils.style import rconsole


class Environment:

    def __init__(self):
        self.verbose = False
        self.config = {}

    def log(self, msg, *args, **kwargs):
        """Logs a message to stderr."""
        if kwargs:
            if kwargs.get('pager') == True:
                with rconsole.pager():
                    rconsole.print(msg)
        else:
            rconsole.print(msg, style='bold magenta')

    def vlog(self, msg, *args):
        """Logs a message to stderr only if verbose is enabled."""
        if self.verbose:
            self.log("DEBUG: {}".format(msg), *args)


pass_environment = click.make_pass_decorator(Environment, ensure=True)


class HoruzCLI(click.MultiCommand):
    def list_commands(self, ctx):
        rv = []
        for filename in os.listdir(os.path.abspath(os.path.join(os.path.dirname(__file__), "commands"))):
            if filename.endswith(".py") and filename.startswith("cmd_"):
                rv.append(filename[4:-3])
        rv.sort()
        return rv

    def get_command(self, ctx, name):
        try:
            mod = __import__(f"horuz.commands.cmd_{name}", None, None, ["cli"])
        except ImportError:
            return
        return mod.cli


@click.command(cls=HoruzCLI)
@pass_environment
def cli(ctx):
    """
    Horuz!. CLI to interact with ElasticSearch. Save and query your recon data on ElasticSearch.
    """
    filename = os.path.expanduser("~/.horuz/horuz.cfg")
    es_address = "http://localhost:9200"
    if os.path.exists(filename):
        with open(filename) as cfg:
            es_address = cfg.read()

    ctx.config = {
        'elasticsearch_address': es_address,
        'config_file': filename
    }



import os
import subprocess
import sys
import time

from horuz.utils.style import rconsole


def execute_command(cmd):
    """
    Function which help us to Execute OS commands
    """
    executed = False
    try:
        with rconsole.status("Executing command..."):
            command = subprocess.Popen(
                cmd,
                shell=True,)
            command.wait()
            executed = not bool(command.returncode)
    except KeyboardInterrupt:
        command.kill()
        # Give time to the command execution to terminate correctly
        time.sleep(1)
    return executed


def log_session(session):
    """
    Saves the name of the given session in a local log
    getting sure not to store duplicates
    """
    os.popen("mkdir ~/.horuz/ 2>/dev/null")
    log_path = '{}/.horuz/sessions.log'.format(os.path.expanduser("~"))

    try:
        with open(log_path) as f:
            lines = f.readlines()
            lines = [x.strip() for x in lines if x]
            if session not in lines:
                lines.append(session)
                with open(log_path, 'w') as f:
                    f.write("\n".join(lines))
    except Exception:
        os.popen("touch ~/.horuz/sessions.log")


def get_sessions(ctx, args, incomplete):
    """
    Used to autocomplete value for session param on collect
    """
    os.popen("mkdir ~/.horuz/ 2>/dev/null")
    log_path = '{}/.horuz/sessions.log'.format(os.path.expanduser("~"))

    try:
        with open(log_path) as f:
            lines = f.readlines()
    except Exception:
        os.popen("touch ~/.horuz/sessions.log 2>/dev/null")
        return []

    return [k.strip() for k in lines if k and incomplete in k]



from collections import abc
import datetime
import json

import click
from elasticsearch import Elasticsearch, RequestsHttpConnection
from elasticsearch.exceptions import RequestError, ConnectionError, ConnectionTimeout
from rich.progress import track

from horuz.utils.generators import get_random_name, get_duplications


class ElasticSearchAPI:
    """
    Interaction with our Elasticsearch server
    """

    def __init__(self, address, ctx):
        """
        Interact with ES.
        Parameters
        ----------
        address : string
            ElasticSearch Address
        ctx : Environment Class
            cli env class
        """
        try:
            self.es = Elasticsearch(
                address, connection_class=RequestsHttpConnection)
        except (ConnectionError, ConnectionTimeout):
            self.ctx.log("Error init connection ES")
        except Exception as e:
            self.ctx.log("Error init ES {}".format(e))
            self.es = None
        self.ctx = ctx

    def create_index(self, name):
        """
        Create the index in our ES Server.
        Parameters
        ----------
        name : String
            Index Name
        Returns
        -------
        boolean
            created or not
        """
        created = False
        try:
            if not self.es.indices.exists(name):
                self.es.indices.create(index=name, ignore=400)
            created = True
        except (ConnectionError, ConnectionTimeout):
            self.ctx.log("Create index connection error")
        except Exception as e:
            self.ctx.log("Create index error {}".format(e))
        finally:
            return created

    def delete_index(self, index):
        """
        Delete the index
        Parameters
        ----------
        Index : String
            Index name
        Returns
        -------
        boolean
            deleted or not
        """
        deleted = False
        try:
            self.es.indices.delete(index=index, ignore=[400, 404])
            deleted = True
        except Exception as e:
            self.ctx.log("Delete index error {}".format(e))
        finally:
            return deleted

    def save_in_index(self, index, record):
        """
        Save the information in the specified index.
        Parameters
        ----------
        index : String
            Index Name
        record : json
            The information we want to save
        Returns
        -------
        boolean
            saved or not
        """
        # Validate if the index exists. If not create
        self.create_index(index)
        saved = False
        try:
            saved = self.es.index(index=index, body=record)
        except (ConnectionError, ConnectionTimeout):
            self.ctx.log("Save index connection error")
        except Exception as e:
            self.ctx.log("Save index error {}".format(e))
        finally:
            return saved

    def get_all_indexes(self):
        """
        Get all Indexes in ElasticSeach
        """
        return self.es.indices.get_alias()

    def get_index_mapping(self, index):
        """
        Get the index mappping
        Parameters
        ----------
        index : String
            Index Name
        Returns
        -------
        json
        """
        try:
            return self.es.indices.get_mapping(index)
        except Exception as e:
            self.ctx.vlog("Mapping error {}".format(e))

    def query(self, index, term, size=100, order="time:desc", raw=False, fields=[]):
        """
        Search in Elasticsearch server
        Parameters
        ----------
        index : String
            Index Name
        term : String
            Search Query
        size : String
            Size query search
        order : String
            Sort by
        fields : List
            A list of fields of the source
        """
        self.create_index(index)
        if raw is False:
            self.ctx.vlog("ElasticSeach Lucene: {}".format(term))
            if term:
                try:
                    return self.es.search(
                        index=index,
                        q=term,
                        sort=[order],
                        size=size,
                        _source=fields)
                except (RequestError, ConnectionError, ConnectionTimeout) as e:
                    self.ctx.vlog("Query Error {}".format(e))
        else:
            search_args = {"index": index, "body": term}
            self.ctx.vlog("ElasticSeach Query Raw: {}".format(search_args))
            try:
                return self.es.search(**search_args)
            except (RequestError, ConnectionError, ConnectionTimeout) as e:
                self.ctx.vlog("Query Error {}".format(e))

    def connected(self):
        try:
            self.es.cluster.health()
            return True
        except Exception:
            return False


class HoruzES:
    """
    Horuz ElasticSearch connection
    """
    def __init__(self, domain, ctx=None):
        self.es = ElasticSearchAPI(ctx.config.get("elasticsearch_address"), ctx)
        self.domain = domain
        self.ctx = ctx

    def save_ffuf_data(self, data, session, filter_dups=None, remove_filter_dups=None):
        """
        Save ffuf data to ES
        Parameters
        ----------
        data : Dict
            Json data
        session : String
            Session name
        filter_dups: String
            field name which is going to be filtered
        """
        session = session if session else get_random_name()
        config_url = data["config"]["url"].replace("FUZZ", "")
        # If data is saved, we do not save it again
        record_exists = self.es.query(
            index=self.domain,
            term='''
                host: "*{}" AND time: {} AND type: ffuf
            '''.format(
                config_url.replace("/", '').replace("http:", ''),
                data["time"]))
        if record_exists and record_exists['hits']['hits']:
            self.ctx.vlog("Record {} {} exists: ", config_url, data["time"], record_exists)
            return
        # Save the new data
        all_es_data = []
        es_data = {
            "host": config_url,
            "time": data.get("time"),
            "type": "ffuf",
            "session": session,
            "cmd": data.get("commandline"),
            "result": []
        }
        # Save the new data
        len_results = len(data.get("results"))
        if data.get("results"):
            for result in track(data["results"], description="Collecting HTML for the session {}...".format(session)):
                es_data["result"] = result
                # Get request/response data
                es_data["result"]["html"] = ""
                if "outputdirectory" in data["config"] and data["config"]["outputdirectory"]:
                    try:
                        with open("{}/{}".format(data["config"]["outputdirectory"], result["resultfile"]), encoding="utf-8", errors="ignore") as f:
                            es_data["result"]["html"] = f.read()
                    except FileNotFoundError:
                        self.ctx.vlog("Could not open file")
                self.ctx.vlog(es_data)
                if filter_dups:
                    all_es_data.append(es_data["result"])
                else:
                    self.es.save_in_index(self.domain, es_data)
            if filter_dups:
                all_es_data = get_duplications(
                    data=all_es_data,
                    filter_dups=filter_dups,
                    remove_filter_dups=remove_filter_dups)
                for result in track(all_es_data, description="Collecting data for the session {}...".format(session)):
                    data_dup = {
                        "host": config_url,
                        "time": data.get("time"),
                        "session": session,
                        "type": "ffuf",
                        "cmd": data.get("commandline"),
                        "result": []
                    }
                    for result in results:
                        # Remove duplicates before to save in ES
                        dups = {}
                        if result.get("dups"):
                            dups = result["dups"]
                            del result["dups"]
                            # Saving to ES
                            data_dup["result"] = result
                            object_es = self.es.save_in_index(self.domain, data_dup)
                            data_dup["duplicate_reference_id"] = object_es["_id"]
                            # Save the reference of the duplicates
                            for dup in dups:
                                data_dup["result"] = dup
                                self.es.save_in_index(self.domain, data_dup)
                            del data_dup["duplicate_reference_id"]
                        else:
                            # Saving to ES
                            data_dup["result"] = result
                            object_es = self.es.save_in_index(self.domain, data_dup)
                        data_dup["result"] = []

        else:
            self.ctx.vlog(es_data)
            self.es.save_in_index(self.domain, es_data)
        self.ctx.log("Project name: [bold deep_pink2]{}[/bold deep_pink2]".format(self.domain))
        self.ctx.log("Session name: [bold deep_pink2]{}[/bold deep_pink2]".format(session))
        self.ctx.log("Results: {}".format(len_results))
        return

    def save_general_data(self, data, session, filter_dups=None, remove_filter_dups=None):
        """
        Save General JSON data
        Parameters
        ----------
        data : Dict
            Json data
        session : String
            Session name
        filter_dups: String
            field name which is going to be filtered
        """
        session = session if session else get_random_name()
        # Filter the duplicate data that is in the JSON
        if filter_dups:
            data = get_duplications(
                data=data,
                filter_dups=filter_dups,
                remove_filter_dups=remove_filter_dups)
        for result in track(data, description="Uploading..."):
            # Adding time and session
            result.update({
                "time": datetime.datetime.now(),
                "session": session})
            # Remove duplicates before to save in ES
            dups = {}
            if "dups" in result:
                dups = result["dups"]
                del result["dups"]
            self.ctx.vlog(result)
            object_es = self.es.save_in_index(self.domain, result)
            # Save the reference of the duplicates
            for dup in dups:
                dup.update({
                    "time": datetime.datetime.now(),
                    "session": session,
                    "{}_duplicate_reference_id".format(
                        filter_dups.replace(".", "_")): object_es["_id"]})
                self.es.save_in_index(self.domain, dup)
        self.ctx.log("Project name: [bold deep_pink2]{}[/bold deep_pink2]".format(self.domain))
        self.ctx.log("Session name: [bold deep_pink2]{}[/bold deep_pink2]".format(session))
        self.ctx.log("Results: {}".format(len(data)))

    def save_json(self, files, session, filter_dups=None, remove_filter_dups=None):
        """
        Save JSON Data to ES.
        Parameters
        ----------
        files : List
            List of files
        session : String
            Session's name
        filter_dups : String
            Filter duplicates by X field
        """
        if self.es.connected() is False:
            self.ctx.log("ElasticSearch connection error")
            return

        for filepath in files:
            with open(filepath, "r") as fp:
                data = {}
                try:
                    data = json.load(fp)
                except json.decoder.JSONDecodeError:
                    self.ctx.vlog("Error decoding the JSON Data")
                    continue
                # Put the data in ES
                if "ffuf" in str(data):
                    self.save_ffuf_data(
                        data, session, filter_dups, remove_filter_dups)
                else:
                    self.save_general_data(
                        data, session, filter_dups, remove_filter_dups)
        return

    def query(self, term, size=100, order="time:desc", raw=False, fields=[]):
        """
        Send Queries to ES
        """
        q = None
        self.ctx.vlog("Sending the query '{}' to ElasticSeach.".format(term))
        try:
            q = self.es.query(self.domain, term, size, order, raw, fields)
        except Exception as e:
            self.ctx.log("Query connection failed: {}!".format(e))
            self.ctx.vlog("{}!".format(e))

        return q

    def delete(self):
        """
        Delete and Index from ES
        """
        d = None
        try:
            d = self.es.delete_index(self.domain)
        except Exception:
            self.ctx.log("Query connection failed!")
        return d

    def indexes(self):
        """
        Get all Indexes from ES.
        """
        s = None
        try:
            s = sorted(self.es.get_all_indexes().keys())
        except Exception:
            self.ctx.log("Query connection failed!")
        return s

    def project_mapping(self):
        """
        Get the mapping indexes
        """
        mapping = []
        try:
            props = self.es.get_index_mapping(self.domain)
            if props:
                for p in props[self.domain]["mappings"]["properties"].keys():
                    if "properties" in props[self.domain]["mappings"]["properties"][p].keys():
                        for pin in props[self.domain]["mappings"]["properties"][p]["properties"].keys():
                            if "properties" in props[self.domain]["mappings"]["properties"][p]["properties"][pin].keys():
                                for pin2 in props[self.domain]["mappings"]["properties"][p]["properties"][pin]['properties'].keys():
                                    mapping.append("{}.{}.{}".format(p, pin ,pin2))
                            else:
                                mapping.append("{}.{}".format(p, pin))
                    else:
                        mapping.append(p)
        except Exception as e:
            self.ctx.log("Mapping connection failed! {}".format(e))
        return mapping

    def is_connected(self):
        """
        Check if ES is connected
        """
        return self.es.connected()



import click

from horuz.cli import pass_environment
from horuz.utils.es import HoruzES
from horuz.utils.style import rtable


@click.group()
def cli():
    """
    Manage your Sessions
    """


@cli.command("ls")
@click.option("-v", "--verbose", is_flag=True, help="Enables verbose mode.")
@click.option('-p', '--project', required=True, help='Specify the project.')
@pass_environment
def sessions_ls(ctx, verbose, project):
    """
    List all your sessions
    """
    ctx.verbose = verbose
    hes = HoruzES(project, ctx)
    data = hes.query(term="""
        {
            "size": 0,
            "aggs" : {
                "sessions" : {
                    "terms" : { "field" : "session.keyword", "size": 1000 }
                }
            }
        }
        """, raw=True)
    rtable.add_column("Session", style="cyan", no_wrap=True)
    rtable.add_column("Count", style="cyan")
    for i in data["aggregations"]["sessions"]["buckets"]:
        rtable.add_row(i['key'], str(i['doc_count']))
    ctx.log(rtable)



import click

from horuz.cli import pass_environment
from horuz.utils.es import HoruzES
from horuz.utils.style import rtable


@click.group()
def cli():
    """
    Manage your ElasticSeach Projects
    """
    pass


@cli.command("rm")
@click.option("-v", "--verbose", is_flag=True, help="Enables verbose mode.")
@click.option('-p', '--project', required=True, help='Specify the project to delete.')
@pass_environment
def projects_rm(ctx, verbose, project):
    """
    Delete ElasticSeach Project
    """
    ctx.verbose = verbose
    click.confirm(
        "Are you sure you want to delete {}?".format(project),
        abort=True,
        default=True)
    # Delete the Index Project
    hes = HoruzES(project, ctx)
    deleted = hes.delete()
    if deleted:
        ctx.log("Project {} was deleted.".format(project))


@cli.command("ls")
@click.option("-v", "--verbose", is_flag=True, help="Enables verbose mode.")
@pass_environment
def projects_ls(ctx, verbose):
    """
    List all your ElasticSearch Projects
    """
    ctx.verbose = verbose
    hes = HoruzES("", ctx)
    indexes = hes.indexes()
    if indexes:
        rtable.add_column("Projects", style="cyan", no_wrap=True)
        for i in indexes:
            rtable.add_row(i)
        ctx.log(rtable)


@cli.command("describe")
@click.option("-v", "--verbose", is_flag=True, help="Enables verbose mode.")
@click.option('-p', '--project', required=True, help='Specify the project to delete.')
@pass_environment
def projects_describe(ctx, verbose, project):
    """
    Project fields
    """
    ctx.verbose = verbose
    hes = HoruzES(project, ctx)
    mapping = hes.project_mapping()
    if mapping:
        rtable.add_column("{} fields".format(project), style="cyan", no_wrap=True)
        for i in mapping:
            rtable.add_row(i)
        ctx.log(rtable)
    else:
        ctx.log("Project does not exist!")



import os


def collect(path=None, prefix=None):
    """
    Collect files from directory.
    """
    if not path:
        raise ValueError("No path is loaded.")
    files = []
    for r, d, f in os.walk(path):
        for file in f:
            if prefix and prefix in file:
                files.append(os.path.join(r, file))
    return files



import random
import time

import click
import dpath.util


def get_random_name():
    """
    Name generator which will be using for the sessions.
    Based on Docker name generation.
    https://github.com/moby/moby/blob/master/pkg/namesgenerator/names-generator.go
    """
    left = [
        "admiring",
        "adoring",
        "affectionate",
        "agitated",
        "amazing",
        "angry",
        "awesome",
        "beautiful",
        "blissful",
        "bold",
        "boring",
        "brave",
        "busy",
        "charming",
        "clever",
        "cool",
        "compassionate",
        "competent",
        "condescending",
        "confident",
        "cranky",
        "crazy",
        "dazzling",
        "determined",
        "distracted",
        "dreamy",
        "eager",
        "ecstatic",
        "elastic",
        "elated",
        "elegant",
        "eloquent",
        "epic",
        "exciting",
        "fervent",
        "festive",
        "flamboyant",
        "focused",
        "friendly",
        "frosty",
        "funny",
        "gallant",
        "gifted",
        "goofy",
        "gracious",
        "great",
        "happy",
        "hardcore",
        "heuristic",
        "hopeful",
        "hungry",
        "infallible",
        "inspiring",
        "interesting",
        "intelligent",
        "jolly",
        "jovial",
        "keen",
        "kind",
        "laughing",
        "loving",
        "lucid",
        "magical",
        "mystifying",
        "modest",
        "musing",
        "naughty",
        "nervous",
        "nice",
        "nifty",
        "nostalgic",
        "objective",
        "optimistic",
        "peaceful",
        "pedantic",
        "pensive",
        "practical",
        "priceless",
        "quirky",
        "quizzical",
        "recursing",
        "relaxed",
        "reverent",
        "romantic",
        "sad",
        "serene",
        "sharp",
        "silly",
        "sleepy",
        "stoic",
        "strange",
        "stupefied",
        "suspicious",
        "sweet",
        "tender",
        "thirsty",
        "trusting",
        "unruffled",
        "upbeat",
        "vibrant",
        "vigilant",
        "vigorous",
        "wizardly",
        "wonderful",
        "xenodochial",
        "youthful",
        "zealous",
        "zen",
    ]

    right = [
        "albattani",
        "allen",
        "almeida",
        "antonelli",
        "agnesi",
        "archimedes",
        "ardinghelli",
        "aryabhata",
        "austin",
        "babbage",
        "banach",
        "banzai",
        "bardeen",
        "bartik",
        "bassi",
        "beaver",
        "bell",
        "benz",
        "bhabha",
        "bhaskara",
        "black",
        "blackburn",
        "blackwell",
        "bohr",
        "booth",
        "borg",
        "bose",
        "bouman",
        "boyd",
        "brahmagupta",
        "brattain",
        "brown",
        "buck",
        "burnell",
        "cannon",
        "carson",
        "cartwright",
        "carver",
        "cerf",
        "chandrasekhar",
        "chaplygin",
        "chatelet",
        "chatterjee",
        "chebyshev",
        "cohen",
        "chaum",
        "clarke",
        "colden",
        "cori",
        "cray",
        "curran",
        "curie",
        "darwin",
        "davinci",
        "dewdney",
        "dhawan",
        "diffie",
        "dijkstra",
        "dirac",
        "driscoll",
        "dubinsky",
        "easley",
        "edison",
        "einstein",
        "elbakyan",
        "elgamal",
        "elion",
        "ellis",
        "engelbart",
        "euclid",
        "euler",
        "faraday",
        "feistel",
        "fermat",
        "fermi",
        "feynman",
        "franklin",
        "gagarin",
        "galileo",
        "galois",
        "ganguly",
        "gates",
        "gauss",
        "germain",
        "goldberg",
        "goldstine",
        "goldwasser",
        "golick",
        "goodall",
        "gould",
        "greider",
        "grothendieck",
        "haibt",
        "hamilton",
        "haslett",
        "hawking",
        "hellman",
        "heisenberg",
        "hermann",
        "herschel",
        "hertz",
        "heyrovsky",
        "hodgkin",
        "hofstadter",
        "hoover",
        "hopper",
        "hugle",
        "hypatia",
        "ishizaka",
        "jackson",
        "jang",
        "jemison",
        "jennings",
        "jepsen",
        "johnson",
        "joliot",
        "jones",
        "kalam",
        "kapitsa",
        "kare",
        "keldysh",
        "keller",
        "kepler",
        "khayyam",
        "khorana",
        "kilby",
        "kirch",
        "knuth",
        "kowalevski",
        "lalande",
        "lamarr",
        "lamport",
        "leakey",
        "leavitt",
        "lederberg",
        "lehmann",
        "lewin",
        "lichterman",
        "liskov",
        "lovelace",
        "lumiere",
        "mahavira",
        "margulis",
        "matsumoto",
        "maxwell",
        "mayer",
        "mccarthy",
        "mcclintock",
        "mclaren",
        "mclean",
        "mcnulty",
        "mendel",
        "mendeleev",
        "meitner",
        "meninsky",
        "merkle",
        "mestorf",
        "mirzakhani",
        "moore",
        "morse",
        "murdock",
        "moser",
        "napier",
        "nash",
        "neumann",
        "newton",
        "nightingale",
        "nobel",
        "noether",
        "northcutt",
        "noyce",
        "panini",
        "pare",
        "pascal",
        "pasteur",
        "payne",
        "perlman",
        "pike",
        "poincare",
        "poitras",
        "proskuriakova",
        "ptolemy",
        "raman",
        "ramanujan",
        "ride",
        "montalcini",
        "ritchie",
        "rhodes",
        "robinson",
        "roentgen",
        "rosalind",
        "rubin",
        "saha",
        "sammet",
        "sanderson",
        "satoshi",
        "shamir",
        "shannon",
        "shaw",
        "shirley",
        "shockley",
        "shtern",
        "sinoussi",
        "snyder",
        "solomon",
        "spence",
        "stonebraker",
        "sutherland",
        "swanson",
        "swartz",
        "swirles",
        "taussig",
        "tereshkova",
        "tesla",
        "tharp",
        "thompson",
        "torvalds",
        "tu",
        "turing",
        "varahamihira",
        "vaughan",
        "visvesvaraya",
        "volhard",
        "villani",
        "wescoff",
        "wilbur",
        "wiles",
        "williams",
        "williamson",
        "wilson",
        "wing",
        "wozniak",
        "wright",
        "wu",
        "yalow",
        "yonath",
        "zhukovsky",
    ]
    name = "{}_{}_{}".format(
        random.choice(left),
        random.choice(right),
        "{}".format(time.time()).split(".")[1])
    return name


def get_duplications(data, filter_dups, remove_filter_dups=None):
    """
    Find the duplications and put them in the same JSON dict in the dups key.
    data : JSON data
        The data.
    filter_dups : String separated with commas
        The fields the user wants to filter/eliminate duplicates
    remove_filter_dups : String separated with commas
        The fileds the user does not want to add in the ends array.
    """
    new_data = []
    filter_dups = tuple(filter_dups.replace(".", "/").split(","))
    if remove_filter_dups:
        remove_filter_dups = remove_filter_dups.replace(".", "/").split(",")
    data_progressbar = data.copy()
    with click.progressbar(data_progressbar, label="Removing duplicates...") as datas:
        for idx, d in enumerate(datas):
            # Iterate each result with all the data
            new_dict = dict(d)
            new_dict["dups"] = []
            # Generate a long string of multiple fields
            f1 = ""
            for filter_dup in filter_dups:
                f1 += "{}".format(str(dpath.util.get(d, filter_dup)))
            for d2 in data[idx + 1:]:
                # Generate a long string of multiple fields
                f2 = ""
                for filter_dup in filter_dups:
                    f2 += "{}".format(str(dpath.util.get(d2, filter_dup)))
                if f1 == f2:
                    if remove_filter_dups:
                        # If the filter is specified, we remove the fields,
                        # if the filter is not specified, it will no be added in the dict
                        dup_dict = dict(d2)
                        for remove_filter_dup in remove_filter_dups:
                            dpath.util.delete(dup_dict, remove_filter_dup)
                        new_dict["dups"].append(dup_dict)
                    data.remove(d2)
            new_data.append(new_dict)
    return new_data



import getpass
import os
import time

import click

from horuz.cli import pass_environment
from horuz.utils.cli import execute_command, log_session, get_sessions
from horuz.utils.es import HoruzES
from horuz.utils.files import collect
from horuz.utils.generators import get_random_name


@click.command("collect", short_help="Collect data from external sources")
@click.option("-v", "--verbose", is_flag=True, help="Enables verbose mode.")
@click.option('-p', '--project', required=True, help='Project name. Example yahoo.com uber.com x.com...')
@click.option('-s', '--session', required=False, help="Custom session name", autocompletion=get_sessions)
@click.option('-c', '--cmd', required=False, help='Generate data from external command')
@click.option('-f', '--filename', required=False, type=click.File('r'), help="JSON file")
@click.option('-fd', '--filter-dups', required=False, help="Filter by duplicates. Put the fields separated with commas that are constantly repeated, you will not keep repeated data.")
@click.option('-rfd', '--remove-filter-dups', required=False, help="Only available if -fd is specified. Remove the duplicate fields, save only the data you need, if the option is not specified, the duplicate tuple will be removed. Example usage -rfd html,resultfile")
@pass_environment
def cli(ctx, verbose, project, session, cmd, filename, filter_dups, remove_filter_dups):
    """
    Collect Data from external sources
    """
    ctx.verbose = verbose
    session = session if session else get_random_name()
    log_session(session)
    if cmd and "ffuf" in cmd:
        # Creating all the neccesary paths
        tmp_path = "/tmp/ffuf_{}/{}".format(
            getpass.getuser(), int(time.time()))
        tmp_output = "{}/ffuf_http.json".format(tmp_path)
        os.popen("mkdir -p {} 2>/dev/null".format(tmp_path))
        os.popen("touch {}".format(tmp_output))
        # Adding output location and raw html path
        cmd = "{} -c -o='{}' -od {}".format(cmd, tmp_output, tmp_path)
        executed = execute_command(cmd)
        if executed:
            ctx.log("Command execution done! :sparkles:")
            is_command_done = True
            # Collecting files in ElasticSeach
            ctx.vlog("Getting the JSON Files.")
            ffuf_files = collect(path=tmp_path, prefix="ffuf_http")
            ctx.vlog("Uploading info to ElasticSeach.")
            hes = HoruzES(project, ctx)
            hes.save_json(
                files=ffuf_files,
                session=session,
                filter_dups=filter_dups,
                remove_filter_dups=remove_filter_dups)
            # Deleting remainign files
            os.popen("rm -rf {}".format(tmp_path))
        else:
            ctx.log("Command execution fail! :collision:")
    if filename:
        hes = HoruzES(project, ctx)
        ctx.vlog("Uploading file info to ElasticSeach.")
        hes.save_json(
            files=[filename.name],
            session=session,
            filter_dups=filter_dups,
            remove_filter_dups=remove_filter_dups)



from setuptools import setup, find_packages
from pathlib import Path

version = __import__("horuz").__version__

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()


setup(
    name='horuz',
    version=version,
    description='Save and query your recon data on ElasticSearch.',
    url="https://horuz.misalabs.com",
    author='Misa G.',
    author_email="hi@misalabs.com",
    maintainer='Misa G.',
    maintainer_email='hi@misalabs.com',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='MIT license',
    packages=find_packages(),
    keywords = ['recon', 'elasticsearch'],
    include_package_data=True,
    install_requires=[
        'click==7.1.2',
        'elasticsearch==7.12.0',
        'requests==2.23.0',
        'dpath==2.0.1',
        'rich==9.13.0',
    ],
    entry_points='''
        [console_scripts]
        hz=horuz.cli:cli
    ''',
    classifiers=[
        'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Intended Audience :: Developers',      # Define that your audience are developers
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',   # Again, pick a license
        'Programming Language :: Python :: 3',      #Specify which pyhton versions that you want to support
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.9', 
        'Programming Language :: Python :: 3.10',
    ],
)



import os

import click

from horuz.cli import pass_environment
from horuz.utils.es import HoruzES


@click.group()
def cli():
    """
    Your horuz configuration
    """
    pass


@cli.command("server:add")
@click.option('-a', '--address', help='ElasticSearch Address http://localhost:9200.')
@pass_environment
def config_server_add(ctx, address):
    """
    Add your ElasticSearch server.
    """
    config_file = os.path.expanduser('~/.horuz/horuz.cfg')
    os.makedirs(os.path.dirname(config_file), exist_ok=True)
    address = click.prompt(
        "Please enter the address of your ElasticSearch",
        default=address
    )
    with open(config_file, 'w') as cfg:
        cfg.write(address)
        ctx.log('ElasticSearch is connected now to {}'.format(address))


@cli.command("server:status")
@pass_environment
def config_server_status(ctx):
    """
    Show the server ElasticSearch status connection.
    """
    hes = HoruzES("", ctx)
    if hes.is_connected():
        ctx.log("ElasticSearch is connected to {} successfully!".format(
            ctx.config.get("elasticsearch_address")))
    else:
        ctx.log("ElasticSearch is not connected to {}.".format(
            ctx.config.get("elasticsearch_address")))
